import mathjs from 'mathjs';
import ColumnArray from './ColumnArray/ColumnArray';
import RowArray from './RowArray/RowArray';

export default class DataObject {
  constructor({ columns = [], rows = [], originalData = false }) {
    this.columnArray = new ColumnArray({ columns });
    this.rowArray = new RowArray({ rows });
    this.originalData = originalData;
    console.log('original data: ' + originalData);
    if (originalData) {
      this.evaluateAllComputedColumns();
    }
  }

  // -------------------------- GETTERS ---------------------------- //

  getColumnIndex(columnName) {
    return this.columnArray.getColumnIndex(columnName);
  }

  getMultipleColumnIndex(columnNameList) {
    return this.columnArray.getMultipleColumnIndex(columnNameList);
  }

  getNumberofColumns() {
    return this.columnArray.getNumberofColumns();
  }

  getNumberofRows() {
    return this.rowArray.getNumberofRows();
  }

  getColumnValuesByName(columnName) {
    const columnIndex = this.getColumnIndex(columnName);
    return this.rowArray.getColumnValues(columnIndex);
  }

  // -------------------------- COMPUTED COLUMNS ------------------- //

  evaluateRowComputedColumn(computedColumn, tableRow) {
    const variables = computedColumn.variables;
    const scope = {};
    for (let i = 0, currentVariable, length = variables.length; i < length; i += 1) {
      currentVariable = variables[i];
      const currentColumn = this.columnArray.getColumn(currentVariable);
      const currentVariableValue = tableRow.rowData[currentColumn.index];
      scope[currentVariable] = currentVariableValue;
    }
    console.log(scope);
    const computedValue = computedColumn.expression.eval(scope);
    console.log(computedValue);
    tableRow.addCell(computedColumn.index, computedValue);
  }

  evaluateComputedColumn(computedColumn) {
    console.log('evaluate computed column');
    for (let i = 0, row, length = this.rowArray.rowList.length; i < length; i += 1) {
      row = this.rowArray.rowList[i];
      this.evaluateRowComputedColumn(computedColumn, row);
    }
  }

  evaluateAllComputedColumns() {
    const computedColumns = this.columnArray.getComputedColumns();
    for (let i = 0, length = computedColumns.length; i < length; i += 1) {
      this.evaluateComputedColumn(computedColumns[i]);
    }
  }

  evaluateRowComputedColumns(row) {
    console.log('evaluate row computed columns');
    const computedColumns = this.columnArray.getComputedColumns();
    for (let i = 0, currentColumn, length = computedColumns.length; i < length; i += 1) {
      currentColumn = computedColumns[i];
      console.log('for column: ' + currentColumn.name);
      this.evaluateRowComputedColumn(currentColumn, row);
    }
    const numberOfComputedColumns = computedColumns.length;
    const rowLength = row.rowData.length;
    const lastIndex = rowLength - 1;
    console.log(lastIndex);
    row.rowData.splice(lastIndex, numberOfComputedColumns);
  }

  // -------------------------- MODIFIERS -------------------------- //

  addColumn(name, type, position) {
    this.columnArray.addColumn(name, type, position);
  }

  pushColumn(name, type, expression = '') {
    this.columnArray.pushColumn(name, type, expression);
  }

  addRow(data, position) {
    const row = this.rowArray.addRow(data, position);
    if (this.originalData) {
      this.evaluateRowComputedColumns(row);
    }
  }

  pushRow(data) {
    const row = this.rowArray.pushRow(data);
    if (this.originalData) {
      this.evaluateRowComputedColumns(row);
    }
  }

  removeRow(position) {
    this.rowArray.removeRow(position);
  }

  removeMultipleRows(rowList) {
    this.rowArray.removeMultipleRows(rowList);
  }

  removeCell(rowPosition, cellPosition) {
    this.rowArray.removeCell(rowPosition, cellPosition);
  }

  removeCellInAllRows(cellPosition) {
    this.rowArray.removeCellInAllRows(cellPosition);
  }

  removeMultipleCellsInAllRows(cellPositionList) {
    this.rowArray.removeMultipleCellsInAllRows(cellPositionList);
  }

  removeColumn(columnName) {
    this.columnArray.removeRow(columnName);
  }

  removeMultipleColumns(columnNameList) {
    this.columnArray.removeMultipleColumns(columnNameList);
  }

  editColumnName(oldColumnName, newColumnName) {
    this.columnArray.editColumnName(oldColumnName, newColumnName);
  }

  editCell(rowIndex, columnName, value) {
    const column = this.columnArray.getColumn(columnName);
    if (column.validate(value)) {
      this.rowArray.editCell(rowIndex, column.index, value);
    }
    else {
      // throw (new Error("editCell: Non-valid value entered."));
    }
  }

  addCell(rowPosition, cellPosition, cellValue) {
    this.rowArray.addCell(rowPosition, cellPosition, cellValue);
  }

  pushCell(rowPosition, cellPosition) {
    this.rowArray.pushCell(rowPosition, cellPosition);
  }
}
